@@ -32,10 +32,6 @@
  * must be ciphered, so the parsing finish here */
 #define SSH_FLAG_PARSER_DONE                 0x02
 
-#define SSH_FLAG_STATE_LOGGED                0x04
-
-#define SSH_FLAG_STATE_LOGGED_LUA            0x08
-
 /* MSG_CODE */
 #define SSH_MSG_NEWKEYS                      21
 
@@ -56,6 +52,7 @@ typedef struct SshHeader_ {
     uint32_t pkt_len;
     uint8_t padding_len;
     uint8_t msg_code;
+    uint16_t banner_len;
     uint8_t buf[6];
     uint8_t buf_offset;
     uint8_t flags;
@@ -63,13 +60,26 @@ typedef struct SshHeader_ {
     uint8_t *proto_version;
     uint8_t *software_version;
     uint8_t *banner_buffer;
-    uint16_t banner_len;
 } SshHeader;
 
+enum {
+    SSH_STATE_IN_PROGRESS,
+    SSH_STATE_BANNER_DONE,
+    SSH_STATE_FINISHED,
+};
+
 /** structure to store the SSH state values */
 typedef struct SshState_ {
     SshHeader srv_hdr;
     SshHeader cli_hdr;
+
+    /* specifies which loggers are done logging */
+    uint32_t logged;
+
+    /* bit flags of mpms that have already run */
+    uint64_t mpm_ids;
+
+    DetectEngineState *de_state;
 } SshState;
 
 void RegisterSSHParsers(void);
