@@ -623,7 +623,7 @@ int DecodeIPV4(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, u
 #ifdef UNITTESTS
 
 /** \test IPV4 with no options. */
-int DecodeIPV4OptionsNONETest01(void)
+static int DecodeIPV4OptionsNONETest01(void)
 {
     uint8_t raw_opts[] = { };
     Packet *p = PacketGetFromAlloc();
@@ -639,7 +639,7 @@ int DecodeIPV4OptionsNONETest01(void)
 }
 
 /** \test IPV4 with EOL option. */
-int DecodeIPV4OptionsEOLTest01(void)
+static int DecodeIPV4OptionsEOLTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_EOL, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
@@ -655,7 +655,7 @@ int DecodeIPV4OptionsEOLTest01(void)
 }
 
 /** \test IPV4 with NOP option. */
-int DecodeIPV4OptionsNOPTest01(void)
+static int DecodeIPV4OptionsNOPTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_NOP, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
@@ -671,7 +671,7 @@ int DecodeIPV4OptionsNOPTest01(void)
 }
 
 /** \test IPV4 with RR option. */
-int DecodeIPV4OptionsRRTest01(void)
+static int DecodeIPV4OptionsRRTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_RR, 0x27, 0x08, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -693,7 +693,7 @@ int DecodeIPV4OptionsRRTest01(void)
 }
 
 /** \test IPV4 with RR option (len too large). */
-int DecodeIPV4OptionsRRTest02(void)
+static int DecodeIPV4OptionsRRTest02(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_RR, 0xff, 0x08, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -715,7 +715,7 @@ int DecodeIPV4OptionsRRTest02(void)
 }
 
 /** \test IPV4 with RR option (ptr too large). */
-int DecodeIPV4OptionsRRTest03(void)
+static int DecodeIPV4OptionsRRTest03(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_RR, 0x27, 0xff, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -737,7 +737,7 @@ int DecodeIPV4OptionsRRTest03(void)
 }
 
 /** \test IPV4 with RR option (ptr not in 4 byte increment). */
-int DecodeIPV4OptionsRRTest04(void)
+static int DecodeIPV4OptionsRRTest04(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_RR, 0x27, 0x05, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -759,7 +759,7 @@ int DecodeIPV4OptionsRRTest04(void)
 }
 
 /** \test IPV4 with QS option. */
-int DecodeIPV4OptionsQSTest01(void)
+static int DecodeIPV4OptionsQSTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_QS, 0x08, 0x0d, 0x00, 0xbe, 0xef, 0x00, 0x00
@@ -777,7 +777,7 @@ int DecodeIPV4OptionsQSTest01(void)
 }
 
 /** \test IPV4 with QS option (len too small) */
-int DecodeIPV4OptionsQSTest02(void)
+static int DecodeIPV4OptionsQSTest02(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_QS, 0x07, 0x0d, 0x00, 0xbe, 0xef, 0x00, 0x00
@@ -795,7 +795,7 @@ int DecodeIPV4OptionsQSTest02(void)
 }
 
 /** \test IPV4 with TS option. */
-int DecodeIPV4OptionsTSTest01(void)
+static int DecodeIPV4OptionsTSTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_TS, 0x24, 0x0d, 0x01, 0x0a, 0x0a, 0x0a, 0x69,
@@ -817,7 +817,7 @@ int DecodeIPV4OptionsTSTest01(void)
 }
 
 /** \test IPV4 with TS option (ptr too small). */
-int DecodeIPV4OptionsTSTest02(void)
+static int DecodeIPV4OptionsTSTest02(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_TS, 0x24, 0x04, 0x01, 0x0a, 0x0a, 0x0a, 0x69,
@@ -839,7 +839,7 @@ int DecodeIPV4OptionsTSTest02(void)
 }
 
 /** \test IPV4 with TS option (ptr too large). */
-int DecodeIPV4OptionsTSTest03(void)
+static int DecodeIPV4OptionsTSTest03(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_TS, 0x24, 0xff, 0x01, 0x0a, 0x0a, 0x0a, 0x69,
@@ -861,7 +861,7 @@ int DecodeIPV4OptionsTSTest03(void)
 }
 
 /** \test IPV4 with TS option (ptr not valid). */
-int DecodeIPV4OptionsTSTest04(void)
+static int DecodeIPV4OptionsTSTest04(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_TS, 0x24, 0x0a, 0x01, 0x0a, 0x0a, 0x0a, 0x69,
@@ -883,7 +883,7 @@ int DecodeIPV4OptionsTSTest04(void)
 }
 
 /** \test IPV4 with SEC option. */
-int DecodeIPV4OptionsSECTest01(void)
+static int DecodeIPV4OptionsSECTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_SEC, 0x0b, 0xf1, 0x35, 0x00, 0x00, 0x00, 0x00,
@@ -902,7 +902,7 @@ int DecodeIPV4OptionsSECTest01(void)
 }
 
 /** \test IPV4 with SEC option (invalid length). */
-int DecodeIPV4OptionsSECTest02(void)
+static int DecodeIPV4OptionsSECTest02(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_SEC, 0x0a, 0xf1, 0x35, 0x00, 0x00, 0x00, 0x00,
@@ -921,7 +921,7 @@ int DecodeIPV4OptionsSECTest02(void)
 }
 
 /** \test IPV4 with LSRR option. */
-int DecodeIPV4OptionsLSRRTest01(void)
+static int DecodeIPV4OptionsLSRRTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_LSRR, 0x27, 0x08, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -943,7 +943,7 @@ int DecodeIPV4OptionsLSRRTest01(void)
 }
 
 /** \test IPV4 with LSRR option (len too large). */
-int DecodeIPV4OptionsLSRRTest02(void)
+static int DecodeIPV4OptionsLSRRTest02(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_LSRR, 0xff, 0x08, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -965,7 +965,7 @@ int DecodeIPV4OptionsLSRRTest02(void)
 }
 
 /** \test IPV4 with LSRR option (ptr too large). */
-int DecodeIPV4OptionsLSRRTest03(void)
+static int DecodeIPV4OptionsLSRRTest03(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_LSRR, 0x27, 0xff, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -987,7 +987,7 @@ int DecodeIPV4OptionsLSRRTest03(void)
 }
 
 /** \test IPV4 with LSRR option (ptr not in 4 byte increment). */
-int DecodeIPV4OptionsLSRRTest04(void)
+static int DecodeIPV4OptionsLSRRTest04(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_LSRR, 0x27, 0x05, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -1009,7 +1009,7 @@ int DecodeIPV4OptionsLSRRTest04(void)
 }
 
 /** \test IPV4 with CIPSO option. */
-int DecodeIPV4OptionsCIPSOTest01(void)
+static int DecodeIPV4OptionsCIPSOTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_CIPSO, 0x18, 0x00, 0x00, 0x00, 0x05, 0x05, 0x12,
@@ -1029,7 +1029,7 @@ int DecodeIPV4OptionsCIPSOTest01(void)
 }
 
 /** \test IPV4 with SID option. */
-int DecodeIPV4OptionsSIDTest01(void)
+static int DecodeIPV4OptionsSIDTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_SID, 0x04, 0xbe, 0xef, 0x00, 0x00, 0x00, 0x00
@@ -1047,7 +1047,7 @@ int DecodeIPV4OptionsSIDTest01(void)
 }
 
 /** \test IPV4 with SID option (len invalid. */
-int DecodeIPV4OptionsSIDTest02(void)
+static int DecodeIPV4OptionsSIDTest02(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_SID, 0x05, 0xbe, 0xef, 0x00, 0x00, 0x00, 0x00
@@ -1065,7 +1065,7 @@ int DecodeIPV4OptionsSIDTest02(void)
 }
 
 /** \test IPV4 with SSRR option. */
-int DecodeIPV4OptionsSSRRTest01(void)
+static int DecodeIPV4OptionsSSRRTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_SSRR, 0x27, 0x08, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -1087,7 +1087,7 @@ int DecodeIPV4OptionsSSRRTest01(void)
 }
 
 /** \test IPV4 with SSRR option (len too large). */
-int DecodeIPV4OptionsSSRRTest02(void)
+static int DecodeIPV4OptionsSSRRTest02(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_SSRR, 0xff, 0x08, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -1109,7 +1109,7 @@ int DecodeIPV4OptionsSSRRTest02(void)
 }
 
 /** \test IPV4 with SSRR option (ptr too large). */
-int DecodeIPV4OptionsSSRRTest03(void)
+static int DecodeIPV4OptionsSSRRTest03(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_SSRR, 0x27, 0xff, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -1131,7 +1131,7 @@ int DecodeIPV4OptionsSSRRTest03(void)
 }
 
 /** \test IPV4 with SSRR option (ptr not in 4 byte increment). */
-int DecodeIPV4OptionsSSRRTest04(void)
+static int DecodeIPV4OptionsSSRRTest04(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_SSRR, 0x27, 0x05, 0xc0, 0xa8, 0x2a, 0x64, 0x00,
@@ -1153,7 +1153,7 @@ int DecodeIPV4OptionsSSRRTest04(void)
 }
 
 /** \test IPV4 with RTRALT option. */
-int DecodeIPV4OptionsRTRALTTest01(void)
+static int DecodeIPV4OptionsRTRALTTest01(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_RTRALT, 0x04, 0xbe, 0xef, 0x00, 0x00, 0x00, 0x00
@@ -1171,7 +1171,7 @@ int DecodeIPV4OptionsRTRALTTest01(void)
 }
 
 /** \test IPV4 with RTRALT option (len invalid. */
-int DecodeIPV4OptionsRTRALTTest02(void)
+static int DecodeIPV4OptionsRTRALTTest02(void)
 {
     uint8_t raw_opts[] = {
         IPV4_OPT_RTRALT, 0x05, 0xbe, 0xef, 0x00, 0x00, 0x00, 0x00
@@ -1199,7 +1199,8 @@ static int IPV4CalculateValidChecksumtest01(void)
 
     csum = *( ((uint16_t *)raw_ipv4) + 5);
 
-    return (csum == IPV4CalculateChecksum((uint16_t *)raw_ipv4, sizeof(raw_ipv4)));
+    FAIL_IF(IPV4Checksum((uint16_t *)raw_ipv4, sizeof(raw_ipv4), csum) != 0);
+    PASS;
 }
 
 static int IPV4CalculateInvalidChecksumtest02(void)
@@ -1213,13 +1214,14 @@ static int IPV4CalculateInvalidChecksumtest02(void)
 
     csum = *( ((uint16_t *)raw_ipv4) + 5);
 
-    return (csum != IPV4CalculateChecksum((uint16_t *)raw_ipv4, sizeof(raw_ipv4)));
+    FAIL_IF(IPV4Checksum((uint16_t *)raw_ipv4, sizeof(raw_ipv4), csum) == 0);
+    PASS;
 }
 
 /**
  * \test IPV4 defrag and packet recursion level test
  */
-int DecodeIPV4DefragTest01(void)
+static int DecodeIPV4DefragTest01(void)
 {
     uint8_t pkt1[] = {
         0x00, 0x50, 0x56, 0x00, 0x03, 0x05, 0xde, 0xad,
@@ -1343,7 +1345,7 @@ end:
  * \test Don't send IPv4 fragments to the upper layer decoder and
  *       and packet recursion level test.
  */
-int DecodeIPV4DefragTest02(void)
+static int DecodeIPV4DefragTest02(void)
 {
     uint8_t pkt1[] = {
         0x00, 0x50, 0x56, 0x00, 0x03, 0x05, 0xde, 0xad,
@@ -1474,7 +1476,7 @@ end:
 /**
  * \test IPV4 defrag and flow retrieval test.
  */
-int DecodeIPV4DefragTest03(void)
+static int DecodeIPV4DefragTest03(void)
 {
     uint8_t pkt[] = {
         0x00, 0x50, 0x56, 0x00, 0x03, 0x05, 0xde, 0xad,
