@@ -34,10 +34,12 @@ enum {
 };
 
 int DetectEngineContentInspection(DetectEngineCtx *de_ctx, DetectEngineThreadCtx *det_ctx,
-                                  Signature *s, SigMatch *sm,
+                                  const Signature *s, const SigMatchData *smd,
                                   Flow *f,
                                   uint8_t *buffer, uint32_t buffer_len,
                                   uint32_t stream_start_offset,
                                   uint8_t inspection_mode, void *data);
 
+void DetectEngineContentInspectionRegisterTests(void);
+
 #endif /* __DETECT_ENGINE_CONTENT_INSPECTION_H__ */
