@@ -53,10 +53,11 @@
 #include "app-layer-parser.h"
 
 static int DetectFilenameMatch (ThreadVars *, DetectEngineThreadCtx *, Flow *,
-        uint8_t, File *, Signature *, SigMatch *);
-static int DetectFilenameSetup (DetectEngineCtx *, Signature *, char *);
+        uint8_t, File *, const Signature *, const SigMatchCtx *);
+static int DetectFilenameSetup (DetectEngineCtx *, Signature *, const char *);
 static void DetectFilenameRegisterTests(void);
 static void DetectFilenameFree(void *);
+static int g_file_match_list_id = 0;
 
 /**
  * \brief Registration function for keyword: filename
@@ -70,17 +71,27 @@ void DetectFilenameRegister(void)
     sigmatch_table[DETECT_FILENAME].Setup = DetectFilenameSetup;
     sigmatch_table[DETECT_FILENAME].Free  = DetectFilenameFree;
     sigmatch_table[DETECT_FILENAME].RegisterTests = DetectFilenameRegisterTests;
+    sigmatch_table[DETECT_FILENAME].flags = SIGMATCH_QUOTES_OPTIONAL|SIGMATCH_HANDLE_NEGATION;
 
-    DetectAppLayerInspectEngineRegister(ALPROTO_HTTP, SIG_FLAG_TOSERVER,
-            DETECT_SM_LIST_FILEMATCH,
-            DetectFileInspectHttp);
-    DetectAppLayerInspectEngineRegister(ALPROTO_HTTP, SIG_FLAG_TOCLIENT,
-            DETECT_SM_LIST_FILEMATCH,
-            DetectFileInspectHttp);
+    DetectAppLayerInspectEngineRegister("files",
+            ALPROTO_HTTP, SIG_FLAG_TOSERVER, HTP_REQUEST_BODY,
+            DetectFileInspectGeneric);
+    DetectAppLayerInspectEngineRegister("files",
+            ALPROTO_HTTP, SIG_FLAG_TOCLIENT, HTP_RESPONSE_BODY,
+            DetectFileInspectGeneric);
 
-    DetectAppLayerInspectEngineRegister(ALPROTO_SMTP, SIG_FLAG_TOSERVER,
-            DETECT_SM_LIST_FILEMATCH,
-            DetectFileInspectSmtp);
+    DetectAppLayerInspectEngineRegister("files",
+            ALPROTO_SMTP, SIG_FLAG_TOSERVER, 0,
+            DetectFileInspectGeneric);
+
+    DetectAppLayerInspectEngineRegister("files",
+            ALPROTO_NFS, SIG_FLAG_TOSERVER, 0,
+            DetectFileInspectGeneric);
+    DetectAppLayerInspectEngineRegister("files",
+            ALPROTO_NFS, SIG_FLAG_TOCLIENT, 0,
+            DetectFileInspectGeneric);
+
+    g_file_match_list_id = DetectBufferTypeGetByName("files");
 
 	SCLogDebug("registering filename rule option");
     return;
@@ -101,22 +112,16 @@ void DetectFilenameRegister(void)
  * \retval 1 match
  */
 static int DetectFilenameMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx,
-        Flow *f, uint8_t flags, File *file, Signature *s, SigMatch *m)
+        Flow *f, uint8_t flags, File *file, const Signature *s, const SigMatchCtx *m)
 {
     SCEnter();
     int ret = 0;
 
-    DetectFilenameData *filename = (DetectFilenameData *)m->ctx;
+    DetectFilenameData *filename = (DetectFilenameData *)m;
 
     if (file->name == NULL)
         SCReturnInt(0);
 
-    if (file->txid < det_ctx->tx_id)
-        SCReturnInt(0);
-
-    if (file->txid > det_ctx->tx_id)
-        SCReturnInt(0);
-
     if (BoyerMooreNocase(filename->name, filename->len, file->name,
                 file->name_len, filename->bm_ctx) != NULL)
     {
@@ -152,7 +157,7 @@ static int DetectFilenameMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx,
  * \retval filename pointer to DetectFilenameData on success
  * \retval NULL on failure
  */
-static DetectFilenameData *DetectFilenameParse (char *str)
+static DetectFilenameData *DetectFilenameParse (const char *str, bool negate)
 {
     DetectFilenameData *filename = NULL;
 
@@ -163,7 +168,7 @@ static DetectFilenameData *DetectFilenameParse (char *str)
 
     memset(filename, 0x00, sizeof(DetectFilenameData));
 
-    if (DetectContentDataParse ("filename", str, &filename->name, &filename->len, &filename->flags) == -1) {
+    if (DetectContentDataParse ("filename", str, &filename->name, &filename->len) == -1) {
         goto error;
     }
 
@@ -172,6 +177,10 @@ static DetectFilenameData *DetectFilenameParse (char *str)
         goto error;
     }
 
+    if (negate) {
+        filename->flags |= DETECT_CONTENT_NEGATED;
+    }
+
     SCLogDebug("flags %02X", filename->flags);
     if (filename->flags & DETECT_CONTENT_NEGATED) {
         SCLogDebug("negated filename");
@@ -207,12 +216,12 @@ error:
  * \retval 0 on Success
  * \retval -1 on Failure
  */
-static int DetectFilenameSetup (DetectEngineCtx *de_ctx, Signature *s, char *str)
+static int DetectFilenameSetup (DetectEngineCtx *de_ctx, Signature *s, const char *str)
 {
     DetectFilenameData *filename = NULL;
     SigMatch *sm = NULL;
 
-    filename = DetectFilenameParse(str);
+    filename = DetectFilenameParse(str, s->init_data->negated);
     if (filename == NULL)
         goto error;
 
@@ -225,7 +234,7 @@ static int DetectFilenameSetup (DetectEngineCtx *de_ctx, Signature *s, char *str
     sm->type = DETECT_FILENAME;
     sm->ctx = (void *)filename;
 
-    SigMatchAppendSMToList(s, sm, DETECT_SM_LIST_FILEMATCH);
+    SigMatchAppendSMToList(s, sm, g_file_match_list_id);
 
     s->file_flags |= (FILE_SIG_NEED_FILE|FILE_SIG_NEED_FILENAME);
     return 0;
@@ -261,9 +270,9 @@ static void DetectFilenameFree(void *ptr)
 /**
  * \test DetectFilenameTestParse01
  */
-int DetectFilenameTestParse01 (void)
+static int DetectFilenameTestParse01 (void)
 {
-    DetectFilenameData *dnd = DetectFilenameParse("\"secret.pdf\"");
+    DetectFilenameData *dnd = DetectFilenameParse("secret.pdf", false);
     if (dnd != NULL) {
         DetectFilenameFree(dnd);
         return 1;
@@ -274,11 +283,11 @@ int DetectFilenameTestParse01 (void)
 /**
  * \test DetectFilenameTestParse02
  */
-int DetectFilenameTestParse02 (void)
+static int DetectFilenameTestParse02 (void)
 {
     int result = 0;
 
-    DetectFilenameData *dnd = DetectFilenameParse("\"backup.tar.gz\"");
+    DetectFilenameData *dnd = DetectFilenameParse("backup.tar.gz", false);
     if (dnd != NULL) {
         if (dnd->len == 13 && memcmp(dnd->name, "backup.tar.gz", 13) == 0) {
             result = 1;
@@ -293,11 +302,11 @@ int DetectFilenameTestParse02 (void)
 /**
  * \test DetectFilenameTestParse03
  */
-int DetectFilenameTestParse03 (void)
+static int DetectFilenameTestParse03 (void)
 {
     int result = 0;
 
-    DetectFilenameData *dnd = DetectFilenameParse("\"cmd.exe\"");
+    DetectFilenameData *dnd = DetectFilenameParse("cmd.exe", false);
     if (dnd != NULL) {
         if (dnd->len == 7 && memcmp(dnd->name, "cmd.exe", 7) == 0) {
             result = 1;
