@@ -47,6 +47,7 @@
         (f)->probing_parser_toclient_alproto_masks = 0; \
         (f)->flags = 0; \
         (f)->file_flags = 0; \
+        (f)->protodetect_dp = 0; \
         (f)->lastts.tv_sec = 0; \
         (f)->lastts.tv_usec = 0; \
         FLOWLOCK_INIT((f)); \
@@ -55,13 +56,12 @@
         (f)->alproto = 0; \
         (f)->alproto_ts = 0; \
         (f)->alproto_tc = 0; \
-        (f)->de_ctx_id = 0; \
+        (f)->alproto_orig = 0; \
+        (f)->alproto_expect = 0; \
+        (f)->de_ctx_version = 0; \
         (f)->thread_id = 0; \
-        (f)->detect_alversion[0] = 0; \
-        (f)->detect_alversion[1] = 0; \
         (f)->alparser = NULL; \
         (f)->alstate = NULL; \
-        (f)->de_state = NULL; \
         (f)->sgh_toserver = NULL; \
         (f)->sgh_toclient = NULL; \
         (f)->flowvar = NULL; \
@@ -89,6 +89,7 @@
         (f)->probing_parser_toclient_alproto_masks = 0; \
         (f)->flags = 0; \
         (f)->file_flags = 0; \
+        (f)->protodetect_dp = 0; \
         (f)->lastts.tv_sec = 0; \
         (f)->lastts.tv_usec = 0; \
         (f)->protoctx = NULL; \
@@ -98,13 +99,10 @@
         (f)->alproto = 0; \
         (f)->alproto_ts = 0; \
         (f)->alproto_tc = 0; \
-        (f)->de_ctx_id = 0; \
+        (f)->alproto_orig = 0; \
+        (f)->alproto_expect = 0; \
+        (f)->de_ctx_version = 0; \
         (f)->thread_id = 0; \
-        (f)->detect_alversion[0] = 0; \
-        (f)->detect_alversion[1] = 0; \
-        if ((f)->de_state != NULL) { \
-            DetectEngineStateReset((f)->de_state, (STREAM_TOSERVER | STREAM_TOCLIENT)); \
-        } \
         (f)->sgh_toserver = NULL; \
         (f)->sgh_toclient = NULL; \
         GenericVarFree((f)->flowvar); \
@@ -118,9 +116,6 @@
         SC_ATOMIC_DESTROY((f)->use_cnt); \
         \
         FLOWLOCK_DESTROY((f)); \
-        if ((f)->de_state != NULL) { \
-            DetectEngineStateFlowFree((f)->de_state); \
-        } \
         GenericVarFree((f)->flowvar); \
     } while(0)
 
