@@ -49,6 +49,8 @@
 #include "util-logopenfile.h"
 #include "util-time.h"
 
+#include "output-lua.h"
+
 #ifdef HAVE_LUA
 
 #include <lua.h>
@@ -84,7 +86,7 @@ typedef struct LogLuaThreadCtx_ {
     LogLuaCtx *lua_ctx;
 } LogLuaThreadCtx;
 
-static TmEcode LuaLogThreadInit(ThreadVars *t, void *initdata, void **data);
+static TmEcode LuaLogThreadInit(ThreadVars *t, const void *initdata, void **data);
 static TmEcode LuaLogThreadDeinit(ThreadVars *t, void *data);
 
 /** \internal
@@ -211,8 +213,13 @@ static int LuaPacketLoggerAlerts(ThreadVars *tv, void *thread_data, const Packet
 
         lua_getglobal(td->lua_ctx->luastate, "log");
 
+        void *txptr = NULL;
+        if (p->flow && p->flow->alstate && (pa->flags & PACKET_ALERT_FLAG_TX))
+            txptr = AppLayerParserGetTx(p->proto, p->flow->alproto, p->flow->alstate, pa->tx_id);
+
         LuaStateSetThreadVars(td->lua_ctx->luastate, tv);
         LuaStateSetPacket(td->lua_ctx->luastate, (Packet *)p);
+        LuaStateSetTX(td->lua_ctx->luastate, txptr);
         LuaStateSetFlow(td->lua_ctx->luastate, p->flow);
         LuaStateSetPacketAlert(td->lua_ctx->luastate, (PacketAlert *)pa);
 
@@ -236,81 +243,6 @@ static int LuaPacketConditionAlerts(ThreadVars *tv, const Packet *p)
     return FALSE;
 }
 
-/** \internal
- *  \brief Packet Logger for lua scripts, for ssh
- *
- *  A single call to this function will run one script for a single
- *  packet. If it is called, it means that the registered condition
- *  function has returned TRUE.
- *
- *  The script is called once for each packet.
- */
-static int LuaPacketLoggerSsh(ThreadVars *tv, void *thread_data, const Packet *p)
-{
-    LogLuaThreadCtx *td = (LogLuaThreadCtx *)thread_data;
-
-    char timebuf[64];
-    CreateTimeString(&p->ts, timebuf, sizeof(timebuf));
-
-    SCMutexLock(&td->lua_ctx->m);
-
-    lua_getglobal(td->lua_ctx->luastate, "log");
-
-    LuaStateSetThreadVars(td->lua_ctx->luastate, tv);
-    LuaStateSetPacket(td->lua_ctx->luastate, (Packet *)p);
-    LuaStateSetFlow(td->lua_ctx->luastate, p->flow);
-
-    int retval = lua_pcall(td->lua_ctx->luastate, 0, 0, 0);
-    if (retval != 0) {
-        SCLogInfo("failed to run script: %s", lua_tostring(td->lua_ctx->luastate, -1));
-    }
-
-    SCMutexUnlock(&td->lua_ctx->m);
-
-    SshState *ssh_state = (SshState *)FlowGetAppState(p->flow);
-    if (ssh_state != NULL)
-        ssh_state->cli_hdr.flags |= SSH_FLAG_STATE_LOGGED_LUA;
-
-    SCReturnInt(0);
-}
-
-static int LuaPacketConditionSsh(ThreadVars *tv, const Packet *p)
-{
-    if (p->flow == NULL) {
-        return FALSE;
-    }
-
-    if (!(PKT_IS_IPV4(p)) && !(PKT_IS_IPV6(p))) {
-        return FALSE;
-    }
-
-    if (!(PKT_IS_TCP(p))) {
-        return FALSE;
-    }
-
-    uint16_t proto = FlowGetAppProtocol(p->flow);
-    if (proto != ALPROTO_SSH)
-        goto dontlog;
-
-    SshState *ssh_state = (SshState *)FlowGetAppState(p->flow);
-    if (ssh_state == NULL) {
-        SCLogDebug("no ssh state, so no request logging");
-        goto dontlog;
-    }
-
-    if (ssh_state->cli_hdr.software_version == NULL ||
-        ssh_state->srv_hdr.software_version == NULL)
-        goto dontlog;
-
-    /* We only log the state once */
-    if (ssh_state->cli_hdr.flags & SSH_FLAG_STATE_LOGGED_LUA)
-        goto dontlog;
-
-    return TRUE;
-dontlog:
-    return FALSE;
-}
-
 /** \internal
  *  \brief Packet Logger for lua scripts, for packets
  *
@@ -647,8 +579,6 @@ static int LuaScriptInit(const char *filename, LogLuaScriptOptions *options) {
         goto error;
     }
 
-    /* pop the table */
-    lua_pop(luastate, 1);
     LuaReturnState(luastate);
     return 0;
 error:
@@ -759,7 +689,11 @@ static OutputCtx *OutputLuaLogInitSub(ConfNode *conf, OutputCtx *parent_ctx)
     }
 
     char path[PATH_MAX] = "";
-    snprintf(path, sizeof(path),"%s%s%s", dir, strlen(dir) ? "/" : "", conf->val);
+    int ret = snprintf(path, sizeof(path),"%s%s%s", dir, strlen(dir) ? "/" : "", conf->val);
+    if (ret < 0 || ret == sizeof(path)) {
+        SCLogError(SC_ERR_SPRINTF,"failed to construct lua script path");
+        goto error;
+    }
     SCLogDebug("script full path %s", path);
 
     SCMutexLock(&lua_ctx->m);
@@ -861,6 +795,7 @@ static OutputCtx *OutputLuaLogInit(ConfNode *conf)
 
         if (opts.alproto == ALPROTO_HTTP && opts.streaming) {
             om->StreamingLogFunc = LuaStreamingLogger;
+            om->stream_type = STREAMING_HTTP_BODIES;
             om->alproto = ALPROTO_HTTP;
             AppLayerHtpEnableRequestBodyCallback();
             AppLayerHtpEnableResponseBodyCallback();
@@ -883,8 +818,10 @@ static OutputCtx *OutputLuaLogInit(ConfNode *conf)
             AppLayerParserRegisterLogger(IPPROTO_TCP, ALPROTO_DNS);
             AppLayerParserRegisterLogger(IPPROTO_UDP, ALPROTO_DNS);
         } else if (opts.alproto == ALPROTO_SSH) {
-            om->PacketLogFunc = LuaPacketLoggerSsh;
-            om->PacketConditionFunc = LuaPacketConditionSsh;
+            om->TxLogFunc = LuaTxLogger;
+            om->alproto = ALPROTO_SSH;
+            om->tc_log_progress = SSH_STATE_BANNER_DONE;
+            om->ts_log_progress = SSH_STATE_BANNER_DONE;
         } else if (opts.alproto == ALPROTO_SMTP) {
             om->TxLogFunc = LuaTxLogger;
             om->alproto = ALPROTO_SMTP;
@@ -902,6 +839,7 @@ static OutputCtx *OutputLuaLogInit(ConfNode *conf)
             AppLayerHtpNeedFileInspection();
         } else if (opts.streaming && opts.tcp_data) {
             om->StreamingLogFunc = LuaStreamingLogger;
+            om->stream_type = STREAMING_TCP_DATA;
         } else if (opts.flow) {
             om->FlowLogFunc = LuaFlowLogger;
         } else if (opts.stats) {
@@ -949,7 +887,7 @@ static void OutputLuaLogDoDeinit(LogLuaCtx *lua_ctx)
  *
  *  Currently only stores a pointer to the global LogLuaCtx
  */
-static TmEcode LuaLogThreadInit(ThreadVars *t, void *initdata, void **data)
+static TmEcode LuaLogThreadInit(ThreadVars *t, const void *initdata, void **data)
 {
     LogLuaThreadCtx *td = SCMalloc(sizeof(*td));
     if (unlikely(td == NULL))
