@@ -23,6 +23,7 @@
 #include "suricata-common.h"
 #include "config.h"
 #include "util-unittest.h"
+#include "runmode-unittests.h"
 
 #ifdef UNITTESTS
 
@@ -38,7 +39,6 @@
 #include "detect-engine-uri.h"
 #include "detect-engine-hcbd.h"
 #include "detect-engine-hsbd.h"
-#include "detect-engine-hhd.h"
 #include "detect-engine-hrhd.h"
 #include "detect-engine-hmd.h"
 #include "detect-engine-hcd.h"
@@ -94,7 +94,6 @@
 #include "util-magic.h"
 #include "util-memcmp.h"
 #include "util-misc.h"
-#include "util-ringbuffer.h"
 #include "util-signal.h"
 
 #include "reputation.h"
@@ -125,7 +124,6 @@
 
 #endif /* UNITTESTS */
 
-void RegisterAllModules();
 void TmqhSetup (void);
 
 #ifdef UNITTESTS
@@ -148,6 +146,7 @@ static void RegisterUnittests(void)
     HostBitRegisterTests();
     IPPairBitRegisterTests();
     StatsRegisterTests();
+    DecodeEthernetRegisterTests();
     DecodePPPRegisterTests();
     DecodeVLANRegisterTests();
     DecodeRawRegisterTests();
@@ -191,11 +190,9 @@ static void RegisterUnittests(void)
     SCProfilingRegisterTests();
 #endif
     DeStateRegisterTests();
-    DetectRingBufferRegisterTests();
     MemcmpRegisterTests();
     DetectEngineHttpClientBodyRegisterTests();
     DetectEngineHttpServerBodyRegisterTests();
-    DetectEngineHttpHeaderRegisterTests();
     DetectEngineHttpRawHeaderRegisterTests();
     DetectEngineHttpMethodRegisterTests();
     DetectEngineHttpCookieRegisterTests();
@@ -234,7 +231,7 @@ static void RegisterUnittests(void)
  * This function is terminal and will call exit after being called.
  */
 
-void RunUnittests(int list_unittests, char *regex_arg)
+void RunUnittests(int list_unittests, const char *regex_arg)
 {
 #ifdef UNITTESTS
     /* Initializations for global vars, queues, etc (memsets, mutex init..) */
@@ -267,7 +264,6 @@ void RunUnittests(int list_unittests, char *regex_arg)
 #ifdef DBG_MEM_ALLOC
     SCLogInfo("Memory used at startup: %"PRIdMAX, (intmax_t)global_mem);
 #endif
-    SCReputationInitCtx();
     SCProtoNameInit();
 
     TagInitCtx();
@@ -282,7 +278,7 @@ void RunUnittests(int list_unittests, char *regex_arg)
 
     StorageFinalize();
    /* test and initialize the unittesting subsystem */
-    if(regex_arg == NULL){
+    if (regex_arg == NULL){
         regex_arg = ".*";
         UtRunSelftest(regex_arg); /* inits and cleans up again */
     }
