@@ -627,9 +627,10 @@ static int FileAppendDataDo(File *ff, const uint8_t *data, uint32_t data_len)
 
     SCLogDebug("appending %"PRIu32" bytes", data_len);
 
-    if (AppendData(ff, data, data_len) != 0) {
+    int r = AppendData(ff, data, data_len);
+    if (r != 0) {
         ff->state = FILE_STATE_ERROR;
-        SCReturnInt(-1);
+        SCReturnInt(r);
     }
 
     SCReturnInt(0);
@@ -826,17 +827,21 @@ File *FileOpenFile(FileContainer *ffc, const StreamingBufferConfig *sbcfg,
 
     SCReturnPtr(ff, "File");
 }
-File *FileOpenFileWithId(FileContainer *ffc, const StreamingBufferConfig *sbcfg,
+
+/**
+ *  \retval 0 ok
+ *  \retval -1 failed */
+int FileOpenFileWithId(FileContainer *ffc, const StreamingBufferConfig *sbcfg,
         uint32_t track_id, const uint8_t *name, uint16_t name_len,
         const uint8_t *data, uint32_t data_len, uint16_t flags)
 {
     File *ff = FileOpenFile(ffc, sbcfg, name, name_len, data, data_len, flags);
     if (ff == NULL)
-        return NULL;
+        return -1;
 
     ff->file_track_id = track_id;
     ff->flags |= FILE_USE_TRACKID;
-    return ff;
+    return 0;
 }
 
 static int FileCloseFilePtr(File *ff, const uint8_t *data,
@@ -932,6 +937,7 @@ int FileCloseFile(FileContainer *ffc, const uint8_t *data,
 
     SCReturnInt(0);
 }
+
 int FileCloseFileById(FileContainer *ffc, uint32_t track_id,
         const uint8_t *data, uint32_t data_len, uint16_t flags)
 {
@@ -944,10 +950,8 @@ int FileCloseFileById(FileContainer *ffc, uint32_t track_id,
     File *ff = ffc->head;
     for ( ; ff != NULL; ff = ff->next) {
         if (track_id == ff->file_track_id) {
-            if (FileCloseFilePtr(ff, data, data_len, flags) == -1) {
-                SCReturnInt(-1);
-            }
-            SCReturnInt(0);
+            int r = FileCloseFilePtr(ff, data, data_len, flags);
+            SCReturnInt(r);
         }
     }
     SCReturnInt(-1);
