@@ -51,12 +51,17 @@ void TimeModeSetOffline (void);
 int TimeModeIsLive(void);
 
 struct tm *SCLocalTime(time_t timep, struct tm *result);
-void CreateTimeString (const struct timeval *ts, char *str, size_t size);
-void CreateIsoTimeString (const struct timeval *ts, char *str, size_t size);
-void CreateUtcIsoTimeString (const struct timeval *ts, char *str, size_t size);
-time_t SCMkTimeUtc (struct tm *tp);
-int SCStringPatternToTime (char *string, char **patterns,
+void CreateTimeString(const struct timeval *ts, char *str, size_t size);
+void CreateIsoTimeString(const struct timeval *ts, char *str, size_t size);
+void CreateUtcIsoTimeString(const struct timeval *ts, char *str, size_t size);
+void CreateFormattedTimeString(const struct tm *t, const char * fmt, char *str, size_t size);
+time_t SCMkTimeUtc(struct tm *tp);
+int SCStringPatternToTime(char *string, const char **patterns,
                            int num_patterns, struct tm *time);
+int SCTimeToStringPattern (time_t epoch, const char *pattern, char *str,
+                           size_t size);
+uint64_t SCParseTimeSizeString (const char *str);
+uint64_t SCGetSecondsUntil (const char *str, time_t epoch);
 
 #endif /* __UTIL_TIME_H__ */
 
